﻿<#

.SYNOPSIS
    Collects information from legacy systems and users for input into the discovery database

.DESCRIPTION
    Collects mapped printers, mapped drives, email address, and RSO Type for legacy systems and users. Data is output to a csv file per domain in a central location. Designed to run from a group policy.

.OUTPUTS
    2g_ImportCSV_UserEmailAddresses_$($fqdn)_$($userFqdn).csv
    3q_ImportCSV_systems_$($fqdn)_$($userFqdn).csv
    3n_ImportCSV_Printers_rev3_x_Systems_$($fqdn)_$($userFqdn).csv
    3o_ImportCSV_Printers_rev3_x_Users_$($fqdn)_$($userFqdn).csv
    3p_ImportCSV_MappedDrives_$($fqdn)_$($userFqdn).csv
    5a_ImportCSV_Applications_$($fqdn)_$($userFqdn).csv

.EXAMPLE
    Run the script

    .\get-LegacyInformation.ps1

.NOTES
    Created by: Paul Wojtysiak
    Created on: 10/06/2017
    Requirement: Permissions for trusted users to write to the share and file

#>

<#
    %SystemRoot%\system32\WindowsPowerShell\v1.0\powershell.exe -ExecutionPolicy Bypass -NoLogo -NonInteractive -NoProfile -WindowStyle Hidden -file \\files.wustl.edu\shares\IT\PMO\Proj-USI\LegacyInformation\get-LegacyInformation.ps1

    NTFS
    Authenticated Users Read & Execute
    Authenticated Users Create files/write data and Create folders/append data

    Share permissions
    Authenticated Users Change or Full

    Use WMI filter for workstations or:
        DomainRole Responses
        0 Standalone Workstation
        1 Member Workstation
        2 Standalone Server
        3 Member Server
        4 Backup Domain Controller
        5 Primary Domain Controller
        default Not a known Domain Role

        $domainRole = (Get-WmiObject Win32_ComputerSystem).DomainRole
        If ($domainRole -eq 1) {
            Write-Host -ForegroundColor Green "Member Workstation"
            #<put code here for gathering workstation information>
        }
#>

#region Create variables
$fisCode = "003265"

$scriptVersion = "4"
$userName = (Get-ChildItem Env:\USERNAME).Value
$userDomain = (Get-ChildItem Env:\USERDNSDOMAIN).Value.ToLower()
$userFqdn = $userDomain+"-"+$userName
$computerName = (Get-WmiObject win32_computersystem).DNSHostName
$computerDomain = (Get-WmiObject win32_computersystem).Domain.ToLower()
$fqdn = $computerName+"."+$computerDomain
$os = (Get-WmiObject -class Win32_OperatingSystem).Caption
$osVersion = (Get-WmiObject -class Win32_OperatingSystem).ServicePackMajorVersion
#$osArchitecture = (Get-WmiObject -class Win32_OperatingSystem).OSArchitecture
$version = ((Get-WmiObject -Class win32_operatingsystem).Version).Split(".")[0]
$sysInfo = get-wmiobject Win32_ComputerSystem -Namespace "root\CIMV2" -ErrorAction Stop
$serialNumber = ((Get-WmiObject Win32_BIOS -Namespace "root\CIMV2").SerialNumber).Replace("VMware-","")
#$os = Get-WmiObject Win32_OperatingSystem -Namespace "root\CIMV2"
$disk = Get-WmiObject Win32_LogicalDisk -Namespace "root\CIMV2" -Filter "DeviceID='$((Get-WmiObject Win32_OperatingSystem).SystemDrive)'"
[array]$procs = Get-WmiObject Win32_Processor -Namespace "root\CIMV2"

$savePath = "\\files.wustl.edu\shares\IT\PMO\Proj-USI\LegacyInformation"
$emailData = Join-Path -Path $savePath -ChildPath "2g_ImportCSV_UserEmailAddresses_$($fqdn)_$($userFqdn).csv"
$machineData = Join-Path -Path $savePath -ChildPath "3q_ImportCSV_systems_$($fqdn)_$($userFqdn).csv"
$printerMachineData = Join-Path -Path $savePath -ChildPath "3n_ImportCSV_Printers_rev3_x_Systems_$($fqdn)_$($userFqdn).csv"
$printerUserData = Join-Path -Path $savePath -ChildPath "3o_ImportCSV_Printers_rev3_x_Users_$($fqdn)_$($userFqdn).csv"
$driveData = Join-Path -Path $savePath -ChildPath "3p_ImportCSV_MappedDrives_$($fqdn)_$($userFqdn).csv"
$applicationData = Join-Path -Path $savePath -ChildPath "5a_ImportCSV_Applications_$($fqdn)_$($userFqdn).csv"
#endregion

#region Configure Registry
if (-not (Test-Path HKCU:\Software\WashUIT\EUS\Scripts\GetLegacyInformation)) {
    New-Item -Path HKCU:\Software\ -Name WashUIT\EUS\Scripts\GetLegacyInformation -Force | Out-Null
}

if (-not (Get-ItemProperty HKCU:\Software\WashUIT\EUS\Scripts\GetLegacyInformation -Name $computerName -ErrorAction SilentlyContinue -WarningAction SilentlyContinue)) {
    Set-ItemProperty -Path HKCU:\Software\WashUIT\EUS\Scripts\GetLegacyInformation -Name $computerName -Value 0 -Force | Out-Null
}

$regUser = Get-ItemProperty -Path HKCU:\Software\WashUIT\EUS\Scripts\GetLegacyInformation -Name $computerName
$regUser.$computerName
#endregion

if ($regUser.$computerName -lt $scriptVersion) {
# Gather User Data
    #region Get Email Address from Reigstry
    $regUserStart = "HKCU:\SOFTWARE\Microsoft\Office"
    $emailAddressesFromRegistry = Get-ChildItem $regUserStart -Recurse | Where-Object {$_.Property -eq "EmailAddress"} | ForEach-Object {$_.GetValue("EmailAddress")} | Sort-Object -Unique | Where-Object {$_ -like "*@wustl.edu"}

    $emailAddresses = foreach ($emailAddressFromRegistry in $emailAddressesFromRegistry) {
        New-Object psobject -Property @{
            'Department Code' = $fisCode
            'WUPeople ID' = ""
            'WUSTL Key' = ""
            EmplID = ""
            'Local Domain Username' = $userName
            'Email Address' = $emailAddressFromRegistry
	    }  | Select-Object 'Department Code', 'WUPeople ID', 'WUSTL Key', EmplID, 'Local Domain Username', 'Email Address'
    }
    #endregion

    #region Printers
    $regex = [regex] "(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b"
    $allPrinters = Get-WmiObject -Class win32_printer

	$printers = foreach ($printer in $allPrinters) {
        $printerIpAddress = $regex.Matches($($printer.PortName)) | ForEach-Object{ $_.value }

        New-Object psobject -Property @{
	        'Printer Name' = $($printer.Name)
            'WUPeople ID' = ""
            'WUSTL Key' = ""
            EmplID = ""
            'Local Domain User' = $userName
	        'Server Name' = $computerName
            'IP Address' = $printerIpAddress
        }  | Select-Object 'Printer Name', 'WUPeople ID', 'WUSTL Key', EmplID, 'Local Domain User', 'Server Name', 'IP Address'
    }
    #endregion

    #region Mapped Drives
    if (Test-Path HKCU:\Network\*) {
        $mappedDrives = Get-Item -Path HKCU:\Network\*
    }
    else {
        $mappedDrives = $null
    }

    if ($mappedDrives) {
        $mappedDrivesOut = foreach ($mappedDrive in $mappedDrives) {
            $letter = $mappedDrive.Name.Split("\")[-1]
            $regPath = "HKCU:\Network\" + $letter
            $mappedDriveLetter = $letter + ":"
            $mappedDrivePath = (Get-ItemProperty -Path $regPath -Name RemotePath).RemotePath

            New-Object psobject -Property @{
                'Department Code' = $fisCode
                'WUPeople ID' = ""
                'WUSTL Key' = ""
                EmplID = ""
                'Local Domain Username' = $userName
                Hostname = $computerName
                'Domain Name' = $computerDomain
                'Drive Letter' = $mappedDriveLetter
                'Share Path' = $mappedDrivePath
            }  | Select-Object 'Department Code', 'WUPeople ID', 'WUSTL Key', EmplID, 'Local Domain Username', Hostname, 'Domain Name', 'Drive Letter', 'Share Path'
        }
    }
    else {
        $mappedDrivesOut = $null
    }
    #endregion

# Gather Machine Data
    #region Get Imprivata Type
    if ((Test-Path HKLM:\SOFTWARE\WOW6432Node\SSOProvider\ISXAgent)) {
        $rsoMachineType = Get-ItemProperty -Path HKLM:\SOFTWARE\WOW6432Node\SSOProvider\ISXAgent -Name Type

        if (($rsoMachineType).Type -eq "1") {
            $rsoName = "RSO 1 - Single"
            $multipleUserMachine = $null
        } ElseIf (($rsoMachineType).Type -eq "2") {
            $rsoName = "RSO 2 - Shared"
            $multipleUserMachine = "True"
        }
    }
    #endregion

    #region Administrators Group Membership
    $Group = [ADSI]("WinNT://localhost/Administrators,group")
    $adminMembers = $Group.Members() | ForEach-Object {
        $AdsPath = $_.GetType().InvokeMember('Adspath', 'GetProperty', $null, $_, $null)
        $A = $AdsPath.split('/',[StringSplitOptions]::RemoveEmptyEntries)
        $Names = $a[-1]
        $Domain = $a[-2]

        $Domain + "\" + $Names
    }
    #endregion

    #region Remote Desktop Users Group Membership
    $Group = [ADSI]("WinNT://localhost/Remote Desktop Users,group")
    $rdsMembers = $Group.Members() | ForEach-Object {
        $AdsPath = $_.GetType().InvokeMember('Adspath', 'GetProperty', $null, $_, $null)
        $A = $AdsPath.split('/',[StringSplitOptions]::RemoveEmptyEntries)
        $Names = $a[-1]
        $Domain = $a[-2]

        $Domain + "\" + $Names
    }
    #endregion

    #region Obtain static IP addresses
    $netAdapters = Get-WmiObject -Class Win32_NetworkAdapter | Where-Object {$_.AdapterType -like "*Ethernet*"}
    foreach ($netAdapter in $netAdapters) {
        if ((Get-WmiObject Win32_NetworkAdapterConfiguration -Namespace "root\CIMV2" | Select-Object IPEnabled, DHCPEnabled, IPAddress, DefaultIPGateway, Description, MACAddress, Index | Where-Object {$_.Index -eq ($netAdapter.DeviceID) -and $_.IPEnabled -eq "True" -and $_.DefaultIPGateway -and $_.DefaultIPGateway -notlike "*:*"})) {
            $primaryIpAddress = (Get-WmiObject Win32_NetworkAdapterConfiguration -Namespace "root\CIMV2" | Select-Object IPEnabled, DHCPEnabled, IPAddress, DefaultIPGateway, Description, MACAddress, Index | Where-Object {$_.Index -eq ($netAdapter.DeviceID) -and $_.IPEnabled -eq "True" -and $_.DefaultIPGateway -and $_.DefaultIPGateway -notlike "*:*"}).IPAddress | Select-Object -First 1
            $wiredMac = (Get-WmiObject Win32_NetworkAdapterConfiguration -Namespace "root\CIMV2" | Select-Object IPEnabled, DHCPEnabled, IPAddress, DefaultIPGateway, Description, MACAddress, Index | Where-Object {$_.Index -eq ($netAdapter.DeviceID) -and $_.IPEnabled -eq "True" -and $_.DefaultIPGateway -and $_.DefaultIPGateway -notlike "*:*"}).MACAddress -replace ":","-"
            if ((Get-WmiObject Win32_NetworkAdapterConfiguration -Namespace "root\CIMV2" | Select-Object IPEnabled, DHCPEnabled, IPAddress, DefaultIPGateway, Description, MACAddress, Index | Where-Object {$_.Index -eq ($netAdapter.DeviceID) -and $_.IPEnabled -eq "True" -and $_.DefaultIPGateway -and $_.DefaultIPGateway -notlike "*:*"})) {
                $networkSpeed = ("$([string]([System.Math]::Round($netAdapter.Speed/1gb,0))) Gbps")
            }
        }
    }

    $staticAddresses = foreach ($netAdapter in $netAdapters) {
        if (-not (Get-WmiObject Win32_NetworkAdapterConfiguration -Namespace "root\CIMV2" | Select-Object IPEnabled, DHCPEnabled, IPAddress, DefaultIPGateway, Description, MACAddress, Index | Where-Object {$_.Index -eq ($netAdapter.DeviceID) -and $_.IPEnabled -eq "True"}).DHCPEnabled) {
            (Get-WmiObject Win32_NetworkAdapterConfiguration -Namespace "root\CIMV2" | Select-Object IPEnabled, DHCPEnabled, IPAddress, DefaultIPGateway, Description, MACAddress, Index | Where-Object {$_.Index -eq ($netAdapter.DeviceID) -and $_.IPEnabled -eq "True"}).IPAddress | Select-Object -First 1
        }
    }
    #endregion

    #region Gather Shares
    $shares = Get-WmiObject -class Win32_Share | Where-Object {$_.Name -ne "ADMIN$" -and $_.Name -ne "C$" -and $_.Name -ne "IPC$" -and $_.Name -ne "print$"} | ForEach-Object { $_.Name + "=" + $_.Path}
    #endregion

    #region Calculate Size of Profile Directory
    $userProfileSum = (Get-ChildItem $env:UserProfile -Recurse -ErrorAction SilentlyContinue -WarningAction SilentlyContinue | Measure-Object -Property Length -Sum).Sum
    #endregion

    #region Determine if Physical or Virtual
    $compsys = Get-WmiObject win32_computersystem
    if($compsys.model.toupper().contains("VIRTUAL"))
    {
        $isPhysical = "False"
        $isVirtual = "True"
    }
    Else {
        $isPhysical = "True"
        $isVirtual = "False"
    }
    #endregion

    #region Dell Warranty information
    if ($PSVersionTable.PSVersion.Major -eq 3 -and $PSVersionTable.PSVersion.Minor -ge 0) {
        #calling sandbox will return same results but there are rate limits
        $apiKey="ef9e41e1-c780-4fbf-8284-a1fd6aa1ce7a"
        #$serviceURL="https://sandbox.api.dell.com/support/assetinfo/v4/getassetsummary"
        $serviceURL="https://api.dell.com/support/assetinfo/v4/getassetsummary"

        # Use a sleep timer to make sure we do not flood the dell system and exceed the rate limit.
        If (($sysInfo.Manufacturer) -like "Dell*") {
            $target = $serviceURL + "/" + $serialNumber + "?apikey=" + $apiKey
            $request = [System.Net.WebRequest]::Create($target)
            $request.Method="Get"
            $request.ContentType="application/json"
            $response = $request.GetResponse()
            $requestStream = $response.GetResponseStream()
            $readStream = New-Object System.IO.StreamReader $requestStream
            $data = $readStream.ReadToEnd()
            $results = $data | ConvertFrom-Json

            #$currItemmodel = $results.AssetSummaryResponse.ProductHeaderData.SystemDescription
            $warranty = ($results.AssetSummaryResponse.AssetEntitlementData | Where-Object -Property EntitlementType -In -Value "EXTENDED")
            if ($null -eq $warranty) {
                $warranty = ($results.AssetSummaryResponse.AssetEntitlementData | Where-Object -Property EntitlementType -In -Value "INITIAL")
            }

            if ($warranty) {
                $validDates = $warranty | Where-Object {$_.ServiceLevelCode -eq "ND"}
                $warrantyStartDate = "{0:MM/dd/yyyy}" -f [datetime]$validDates.StartDate
                $warrantyEndDate = "{0:MM/dd/yyyy}" -f [datetime]$validDates.EndDate
            }

            Start-Sleep -m 5000
        }
    }
    else {
        $warrantyStartDate = $null
        $warrantyEndDate = $null
    }
    #endregion

    #region Build Machine Data File
    $machineInformation = New-Object psobject -Property @{
	    'Department Code' = $fisCode
        'System Type' = "Workstation"
        Hostname = $computerName
        'Hostname New' = ""
        FQDN = $fqdn
        WUPeopleID = ""
        'WUSTL Key' = ""
        EmplID = ""
        'Local Domain Login' = $userName
        'Manager WUSTL Key' = ""
        'Primary IP Address' = $primaryIpAddress
        'Building Name' = "Unknown"
        'Building Floor' = ""
        'Building Room' = ""
        'Location Notes' = ""
        Make = ($sysInfo.Manufacturer)
        Model = ($sysInfo.Model)
        OS = $os
        'OS Version' = (“Service Pack {0}” -f [string]$osVersion)
        'Is Virtual (true/false)' = $isVirtual
        'Is Physical (true/false)' = $isPhysical
        Hypervisor = ""
        'Serial Number' = $serialNumber
        'Num Proc Sockets' = (@($procs).count)
        'Cores Per Socket' = ($procs[0].NumberOfCores)
        'RAM in GB' = ("$([string]([System.Math]::Round($sysInfo.TotalPhysicalMemory/1gb,0,1)))")
        'Total Storage GB' = ("$([string]([System.Math]::Round($disk.Size/1gb,0,1)))")
        'Free Storage GB' = ("$([string]([System.Math]::Round($disk.FreeSpace/1gb,0,1)))")
        Power = ""
        'Warranty Start Date' = $warrantyStartDate
        'Warranty End Date' = $warrantyEndDate
        'Multiple User Workstation (true or false)' = $multipleUserMachine
        'System Description' = ""
        'Notes' = ""
        'Wired MAC' = $wiredMac
        'Network Speed' = $networkSpeed
        'Vendor Supplied (true/false)' = ""
	    'RSO Name' = $rsoName
        'Members - Local Administrators' = ($adminMembers -join '`')
        'Members - Remote Desktop Users' = ($rdsMembers -join '`')
        'Static IP Address' = ($staticAddresses -join '`')
        Shares = ($shares -join '`')
        'Profile Size' = ("$([string]([System.Math]::Round($userProfileSum/1gb,2,1)))")
        system_domain = $computerDomain
        user_domain = $userDomain
        system_grouping = ""
    }  | Select-Object 'Department Code', 'System Type', Hostname, 'Hostname New', FQDN, WUPeopleID, 'WUSTL Key', EmplID, 'Local Domain Login', 'Manager WUSTL Key', 'Primary IP Address', 'Building Name', 'Building Floor', 'Building Room', 'Location Notes', Make, Model, OS, 'OS Version', 'Is Virtual (true/false)', 'Is Physical (true/false)', Hypervisor, 'Serial Number', 'Num Proc Sockets', 'Cores Per Socket', 'RAM in GB', 'Total Storage GB', 'Free Storage GB', Power, 'Warranty Start Date', 'Warranty End Date', 'Multiple User Workstation (true or false)', 'System Description', 'Notes', 'Wired MAC', 'Network Speed', 'Vendor Supplied (true/false)', 'RSO Name', 'Members - Local Administrators', 'Members - Remote Desktop Users', 'Static IP Address', Shares, 'Profile Size', system_domain, user_domain, system_grouping
    #endregion

    #region Collect Application Data
    $32bitWmi = Get-WmiObject -Class Win32_Product | Select-Object -property Name,Vendor,Version
    try {
        $ErrorActionPreference = "Stop"; #Make all errors terminating
        $32bitRegMachine = Get-ItemProperty HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\* | Select-Object DisplayName,Publisher,DisplayVersion
    }
    catch {
    }
    finally {
        $ErrorActionPreference = "Continue"; #Reset the error action pref to default
    }

    try {
        $ErrorActionPreference = "Stop"; #Make all errors terminating
        $64bitRegMachine = Get-ItemProperty HKLM:\SOFTWARE\WOW6432Node\Microsoft\Windows\CurrentVersion\Uninstall\* | Select-Object DisplayName,Publisher,DisplayVersion
    }
    catch {
    }
    finally {
        $ErrorActionPreference = "Continue"; #Reset the error action pref to default
    }

    try {
        $ErrorActionPreference = "Stop"; #Make all errors terminating
        $32bitRegUser = Get-ItemProperty HKCU:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\* | Select-Object DisplayName,Publisher,DisplayVersion
    }
    catch {
    }
    finally {
        $ErrorActionPreference = "Continue"; #Reset the error action pref to default
    }

    $regApps = $32bitRegMachine + $64bitRegMachine + $32bitRegUser
    $regAppsConverted = foreach ($regApp in $regApps) {
        New-Object psobject -Property @{
            'Name' = ($regApp.DisplayName)
            'Vendor' = ($regApp.Publisher)
            'Version' = ($regApp.DisplayVersion)
        } | Select-Object Name, Vendor, Version
    }
    $consolidatedApps = $32bitWmi + $regAppsConverted

    $applications = foreach ($consolidatedApp in $consolidatedApps) {
        if (($consolidatedApp.Name)) {
            New-Object psobject -Property @{
                'Deparment Code' = "999999"
                Hostname = $computerName
                FQDN = $fqdn
                'Application Type' = ""
                'Parent Application Name' = ""
                'Ignore? (true/false)' = "False"
                'Application Name' = ($consolidatedApp.Name)
                Vendor = ($consolidatedApp.Vendor)
                Version = ($consolidatedApp.Version)
                'Product Family' = ""
                'Product Category' = ""
                'Hosted Or Local' = "Local"
                'Is Web Based (true/false)' = "False"
                'Is Client Installed (true/false)' = "True"
            } | Select-Object 'Deparment Code', Hostname, FQDN, 'Application Type', 'Parent Application Name', 'Ignore? (true/false)', 'Application Name', Vendor, Version, 'Product Family', 'Product Category', 'Hosted Or Local', 'Is Web Based (true/false)', 'Is Client Installed (true/false)'
        }
    }
    #endregion

    #region Output
    # Outputs email addresses and legacy username
    do {
        $done = $true
        try {
            $emailAddresses |  Where-Object {$_.'Email Address' -ne $null} | Select-Object 'Department Code', 'WUPeople ID', 'WUSTL Key', EmplID, 'Local Domain Username', 'Email Address' | Sort-Object 'Local Domain Username' | Export-Csv -Path $emailData -Force -NoTypeInformation
        }
        catch {
            $done = $false
            start-sleep -milliseconds $(get-random -min 1000 -max 10000)
        }
    } until ($done)

    # Outputs user printers to department file
    do {
	    $done = $true
	    try {
		    $printers | Select-Object 'Printer Name', 'WUPeople ID', 'WUSTL Key', EmplID, 'Local Domain User', 'IP Address' | Sort-Object 'Printer Name' | Export-Csv -Path $printerUserData -Force -NoTypeInformation
	    }
	    catch {
		    $done = $false
		    start-sleep -milliseconds $(get-random -min 1000 -max 10000)
	    }
    } until ($done)

    # Outputs machine printers to department file
    do {
	    $done = $true
	    try {
		    $printers | Select-Object 'Printer Name', 'Server Name', 'IP Address' | Sort-Object 'Printer Name' | Export-Csv -Path $printerMachineData -Force -NoTypeInformation
	    }
	    catch {
		    $done = $false
		    start-sleep -milliseconds $(get-random -min 1000 -max 10000)
	    }
    } until ($done)

    # Output mapped drives
    do {
	    $done = $true
	    try {
		    $mappedDrivesOut | Where-Object {$_.'Share Path' -notlike "*vbox*"} | Select-Object 'Department Code', 'WUPeople ID', 'WUSTL Key', EmplID, 'Local Domain Username', Hostname, 'Domain Name', 'Drive Letter', 'Share Path' | Sort-Object 'Local Domain Username' | Export-Csv -Path $driveData -Force -NoTypeInformation
	    }
	    catch {
		    $done = $false
		    start-sleep -milliseconds $(get-random -min 1000 -max 10000)
	    }
    } until ($done)

    # Outputs machine discovery data
    do {
	    $done = $true
	    try {
		    $machineInformation | Select-Object 'Department Code', 'System Type', Hostname, 'Hostname New', FQDN, WUPeopleID, 'WUSTL Key', EmplID, 'Local Domain Login', 'Manager WUSTL Key', 'Primary IP Address', 'Building Name', 'Building Floor', 'Building Room', 'Location Notes', Make, Model, OS, 'OS Version', 'Is Virtual (true/false)', 'Is Physical (true/false)', Hypervisor, 'Serial Number', 'Num Proc Sockets', 'Cores Per Socket', 'RAM in GB', 'Total Storage GB', 'Free Storage GB', Power, 'Warranty Start Date', 'Warranty End Date', 'Multiple User Workstation (true or false)', 'System Description', 'Notes', 'Wired MAC', 'Network Speed', 'Vendor Supplied (true/false)', 'RSO Name', 'Members - Local Administrators', 'Members - Remote Desktop Users', 'Static IP Address', Shares, 'Profile Size', system_domain, user_domain, system_grouping | Export-Csv -Path $machineData -Force -NoTypeInformation
	    }
	    catch {
		    $done = $false
		    start-sleep -milliseconds $(get-random -min 1000 -max 10000)
	    }
    } until ($done)

    # Outputs application data
    do {
	    $done = $true
	    try {
            $applications | Group-Object -Property 'Application Name', Vendor, Version | ForEach-Object{$_.Group | Select-Object 'Deparment Code', Hostname, FQDN, 'Application Type', 'Parent Application Name', 'Ignore? (true/false)', 'Application Name', Vendor, Version, 'Product Family', 'Product Category', 'Hosted Or Local', 'Is Web Based (true/false)', 'Is Client Installed (true/false)' -Unique} | Sort-Object 'Application Name' | Export-Csv $applicationData -Force -NoTypeInformation
	    }
	    catch {
		    $done = $false
		    start-sleep -milliseconds $(get-random -min 1000 -max 10000)
	    }
    } until ($done)
    #endregion

    # Set script version number in registry
    Set-ItemProperty -Path HKCU:\Software\WashUIT\EUS\Scripts\GetLegacyInformation -Name $computerName -Value $scriptVersion -Force
}
